/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2007 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.01 of the PHP license,      |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_01.txt                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author: Mayflower GmbH, Barcamp 2008                                 |
  +----------------------------------------------------------------------+
*/

/* $Id: skype.c,v 1.2 2008/01/31 05:18:35 johann Exp $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_skype.h"

/* If you declare any globals in php_skype.h uncomment this:
   ZEND_DECLARE_MODULE_GLOBALS(skype)
*/

/* True global resources - no need for thread safety here */
static int le_skype;

/* {{{ skype_functions[]
 *
 * Every user visible function must have an entry in skype_functions[].
 */
zend_function_entry skype_functions[] = {
	PHP_FE(skype_message,	NULL)
	{NULL, NULL, NULL}	/* Must be the last line in skype_functions[] */
};
/* }}} */

/* {{{ skype_module_entry
 */
zend_module_entry skype_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"skype",
	skype_functions,
	PHP_MINIT(skype),
	PHP_MSHUTDOWN(skype),
	PHP_RINIT(skype),		/* Replace with NULL if there's nothing to do at request start */
	PHP_RSHUTDOWN(skype),	/* Replace with NULL if there's nothing to do at request end */
	PHP_MINFO(skype),
#if ZEND_MODULE_API_NO >= 20010901
	"0.1", /* Replace with version number for your extension */
#endif
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_SKYPE
ZEND_GET_MODULE(skype)
#endif

/* {{{ PHP_INI
 */
/* Remove comments and fill if you need to have entries in php.ini
   PHP_INI_BEGIN()
   STD_PHP_INI_ENTRY("skype.global_value",      "42", PHP_INI_ALL, OnUpdateLong, global_value, zend_skype_globals, skype_globals)
   STD_PHP_INI_ENTRY("skype.global_string", "foobar", PHP_INI_ALL, OnUpdateString, global_string, zend_skype_globals, skype_globals)
   PHP_INI_END()
*/
/* }}} */

/* {{{ php_skype_init_globals
 */
/* Uncomment this function if you have INI entries
   static void php_skype_init_globals(zend_skype_globals *skype_globals)
   {
   skype_globals->global_value = 0;
   skype_globals->global_string = NULL;
   }
*/
/* }}} */

/* {{{ PHP_MINIT_FUNCTION
 */
PHP_MINIT_FUNCTION(skype)
{
	/* If you have INI entries, uncomment these lines 
	   REGISTER_INI_ENTRIES();
	*/
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION
 */
PHP_MSHUTDOWN_FUNCTION(skype)
{
	/* uncomment this line if you have INI entries
	   UNREGISTER_INI_ENTRIES();
	*/
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request start */
/* {{{ PHP_RINIT_FUNCTION
 */
PHP_RINIT_FUNCTION(skype)
{
	return SUCCESS;
}
/* }}} */

/* Remove if there's nothing to do at request end */
/* {{{ PHP_RSHUTDOWN_FUNCTION
 */
PHP_RSHUTDOWN_FUNCTION(skype)
{
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION
 */
PHP_MINFO_FUNCTION(skype)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "skype support", "enabled");
	php_info_print_table_end();

	/* Remove comments if you have entries in php.ini
	   DISPLAY_INI_ENTRIES();
	*/
}
/* }}} */

char *sendDBusMessage ( DBusConnection *connection, char *msg) {
	DBusMessage *message; 
	DBusMessage *reply;
	DBusError error; 

	char *returnbuf; 

	char *tmp;
	int reply_timeout = -1; 


	dbus_error_init (&error); 

	message = dbus_message_new_method_call (
											"com.Skype.API",
											"/com/Skype", 
											"com.Skype.API", 
											"Invoke"); 

	dbus_message_set_auto_start (message, TRUE); 
	if ( !dbus_message_append_args( message,
									DBUS_TYPE_STRING, 
									&msg, 
									DBUS_TYPE_INVALID )) {
		php_error(E_ERROR, "DBusConnection could not be set to auto start %s\n", error.message);
        dbus_error_free (&error); 
        return;
	}
	reply = dbus_connection_send_with_reply_and_block (connection,
													   message,
													   reply_timeout,
													   &error); 
	if (dbus_error_is_set (&error)) {
		php_error(E_ERROR, "Error in send_with_reply_and_block: %sn", error.message); 
		dbus_error_free (&error); 
		return;
	}
	dbus_message_get_args (reply, 
						   &error, 
						   DBUS_TYPE_STRING, 
						   &tmp, 
						   DBUS_TYPE_INVALID); 
	if (dbus_error_is_set (&error)) {
		php_error(E_ERROR, "Error in dbus_message_get_args: %sn", error.message); 
		dbus_error_free (&error); 
        return; 
	}

	returnbuf = estrdup (tmp); 
	dbus_message_unref( message ); 
	dbus_message_unref( reply ); 
	dbus_error_free (&error); 
	return returnbuf; 
}

/* The previous line is meant for vim and emacs, so it can correctly fold and 
   unfold functions in source code. See the corresponding marks just before 
   function definition, where the functions purpose is also documented. Please 
   follow this convention for the convenience of others editing your code.
*/

/* {{{ proto bool skype_message(string profilename, string message)
 */
PHP_FUNCTION(skype_message)
{
	char *profilename = NULL;
	char *message = NULL;
	int argc = ZEND_NUM_ARGS();
	int profilename_len;
	int message_len;
	char *tmp; 
	char *chatid; 
	char *chatmsg; 

	chatmsg = emalloc(255); 

	DBusError error;
	DBusConnection *conn; 

	if (zend_parse_parameters(argc TSRMLS_CC, "ss", &profilename, &profilename_len, &message, &message_len) == FAILURE) 
		return;
	dbus_error_init (&error); 
	conn = dbus_bus_get (DBUS_BUS_SESSION, &error); 
	if (conn == NULL) 
        {
			php_error(E_ERROR, "Could not connect to DBUS"); 
			dbus_error_free (&error); 
			RETURN_FALSE 
				}
	
	tmp = sendDBusMessage(conn, "NAME php"); 
	tmp = sendDBusMessage(conn, "PROTOCOL 2"); 
	sprintf(chatmsg, "CHAT CREATE %s", profilename); 
	tmp = sendDBusMessage(conn, chatmsg); 
 	if (strlen(tmp) < 30) {
		php_error(E_ERROR, "Got bad chat id"); 
		RETURN_FALSE;
	}

	chatid = emalloc( strlen(tmp) - 19); 

	//CHAT CREATE: CHAT #johannhartmann/$thx4nix;5f664a359b1702d4 STATUS DIALOG
	//void *memcpy(void *dest, const void *src, size_t n);
	
	strncpy(chatid, tmp+5, strlen(tmp) - 19); 
	chatid[strlen(tmp) - 19] = '\0'; 
	
	sprintf(chatmsg, "CHATMESSAGE %s %s", chatid, message); 
	php_error(E_WARNING, "Mitteilung: %s", chatmsg); 

	tmp = sendDBusMessage(conn, chatmsg); 
        
        
	dbus_error_free (&error); 

	efree(chatmsg);
	efree(chatid); 
}
/* }}} */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
